/***

The MIT License

Copyright (c) 2018 Teppei Tamra (TAM)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

***/

#ifndef SAORIAPPLICATION_H
#define SAORIAPPLICATION_H

#include <QApplication>
#include <saoridon.h>
#include <saoriaccount.h>
#include <saoricache.h>
#include <QList>
#include <QSettings>
#include <QDir>
#include <QSqlDatabase>
#include <QNetworkAccessManager>

class SaoriApplication : public QApplication
{
    Q_OBJECT
public:
    SaoriApplication(int &argc, char **argv);
    ~SaoriApplication();
    static Saoridon *findInstance(QUrl instance);
    static SaoriAccount *findAccount(QString account);
    static QList<Saoridon *> * getInstanceList();
    static QList<SaoriAccount *> * getAccountList();
    static SaoriApplication * saori();
    QDir cacheDirectory();
    QDir dataDirectory();
    QSqlDatabase * database();
    SaoriCache * cache();
    QSettings * setting();

public:
    QNetworkAccessManager * manager;

protected:
    QSettings *m_config;
    QList<Saoridon *> m_instanceList;
    QList<SaoriAccount *> m_accountList;
    static SaoriApplication *m_self;
    QDir m_cacheDirectory;
    QDir m_dataDirectory;
    QSqlDatabase m_db;
    SaoriCache *m_cache;

protected:
    void loadSettings();
    void saveInstancesSettings();
    void saveAccountsSettings();

public slots:

signals:

};

#endif // SAORIAPPLICATION_H
